from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CustomUrlConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'custom_url'
    verbose_name = _('Custom URL')



import os
from pathlib import Path
from django.core.management.base import BaseCommand, CommandError

from custom_url.models import CustomUrl


class Command(BaseCommand):
    help = 'Create or update urls.py with the urls created in the database'

    def handle(self, *args, **options):
        q = CustomUrl.objects.all()
        try:
            with open(os.path.join(Path(__file__).resolve().parent.parent.parent, 'urls.py'), 'w+') as f:
                f.write(f'from django.urls import path\n')
                f.write(f'from custom_url import views\n\n')
                f.write(f'urlpatterns = [\n')
                for url in q:
                    f.write(f"\tpath('{url.url}', views.CustomUrlView.as_view()),\n")
                f.write(f']\n')
            
            self.stdout.write(self.style.SUCCESS('custom_url urls.py was updated'))

        except:
            self.stderr.write(self.style.ERROR("urls.py couldn't be opened"))
            raise CommandError("urls.py couldn't be opened")


from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from custom_url.models import CustomUrl


class CustomUrlForm(forms.ModelForm):
    url = forms.RegexField(
        label=_('URL'),
        max_length=100,
        regex=r"^[-\w/\.~]+$",
        help_text=_('Example: “/cv/”. Make sure to have leading and trailing slashes.'),
        error_messages={
            "invalid": _('This value must be a valid URL path.'),
        },
    )

    class Meta:
        model = CustomUrl
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self._trailing_slash_required():
            self.fields['url'].help_text = _('Example: “/cv”. Make sure to have a leading slash.')

    def _trailing_slash_required(self):
        return (settings.APPEND_SLASH and 'django.middleware.common.CommonMiddleware' in settings.MIDDLEWARE)

    def clean_url(self):
        url = self.cleaned_data['url']

        if not url.startswith('/'):
            raise ValidationError(_('URL is missing a leading slash.'), code='missing_leading_slash')
        if self._trailing_slash_required() and not url.endswith('/'):
            raise ValidationError(_('URL is missing a trailing slash.'), code='missing_trailing_slash')
        return url


from django.contrib import admin

from custom_url.forms import CustomUrlForm
from custom_url.models import CustomUrl


class CustomUrlAdmin(admin.ModelAdmin):
    form = CustomUrlForm
    list_display = ('url', 'file', 'file_type',)
    list_filter = ('file_type',)

admin.site.register(CustomUrl, CustomUrlAdmin)


# Generated by Django 4.0.4 on 2022-05-20 20:20

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CustomUrl',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(db_index=True, max_length=100, unique=True, verbose_name='Path')),
                ('file', models.FileField(upload_to='custom_url', validators=[django.core.validators.FileExtensionValidator(allowed_extensions=['txt', 'csv', 'doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx', 'gif', 'jpg', 'jpeg', 'png', 'tif', 'tiff', 'svg', 'pdf'])], verbose_name='File')),
                ('file_type', models.CharField(choices=[('text/plain', 'Plain Text'), ('text/csv', 'CSV'), ('application/vnd.ms-excel', 'MS Excel'), ('application/msword', 'MS Word'), ('application/vnd.ms-powerpoint', 'MS PowerPoint'), ('image/gif', 'GIF'), ('image/jpeg', 'JPEG'), ('image/png', 'PNG'), ('image/tiff', 'TIFF'), ('image/svg+xml', 'SVG'), ('application/pdf', 'PDF')], max_length=50, verbose_name='File Type')),
            ],
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import os, sys

github_url = "https://github.com/luciano-im"
twitter_url = "https://twitter.com/luciano_dev"
package_name = "django-custom-url"
package_url = "{}/{}".format(github_url, package_name)
package_path = os.path.abspath(os.path.dirname(__file__))
long_description_file_path = os.path.join(package_path, "README.md")
long_description_content_type = "text/markdown"
long_description = ""
try:
    long_description_file_options = (
        {} if sys.version_info[0] < 3 else {"encoding": "utf-8"}
    )
    with open(long_description_file_path, "r", **long_description_file_options) as f:
        long_description = f.read()
except IOError:
    pass

setup(
    name=package_name,
    packages=find_packages(),
    version=0.2,
    description="A Django app to easily manage custom url linked to static files.",
    long_description=long_description,
    long_description_content_type=long_description_content_type,
    author="Luciano Muñoz",
    author_email="hola@luciano.im",
    url=package_url,
    # download_url="{}/archive/{}.tar.gz".format(package_url, 0.1),
    project_urls={
        "Documentation": "{}#readme".format(package_url),
        "Issues": "{}/issues".format(package_url),
        "Twitter": twitter_url,
    },
    keywords=[
        "django",
        "custom",
        "url",
        "path",
        "file url",
        "file path",
    ],
    install_requires=[
        "Django >= 3.0",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Build Tools",
    ],
    license="MIT",
    include_package_data = True,
)


from django.db import models
from django.core.validators import FileExtensionValidator
from django.utils.translation import gettext_lazy as _

class CustomUrl(models.Model):
    CONTENT_TYPES = [
        ('text/plain', 'Plain Text'),
        ('text/csv', 'CSV'),
        ('application/vnd.ms-excel', 'MS Excel'),
        ('application/msword', 'MS Word'),
        ('application/vnd.ms-powerpoint', 'MS PowerPoint'),
        ('image/gif', 'GIF'),
        ('image/jpeg', 'JPEG'),
        ('image/png', 'PNG'),
        ('image/tiff', 'TIFF'),
        ('image/svg+xml', 'SVG'),
        ('application/pdf', 'PDF'),
    ]
    url = models.CharField(max_length=100, db_index=True, unique=True, verbose_name=_('URL'))
    file = models.FileField(
        upload_to='custom_url',
        validators=[
            FileExtensionValidator(
                allowed_extensions=['txt', 'csv', 'doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx', 'gif', 'jpg', 'jpeg', 'png', 'tif', 'tiff', 'svg', 'pdf']
            )
        ],
        verbose_name=_('File'),
        help_text = _('Allowed format files: Plain Text, CSV, Excel, Word, PowerPoint, GIF, JPEG, PNG, TIFF, SVG, PDF')
    )
    file_type = models.CharField(max_length=50, choices=CONTENT_TYPES, verbose_name=_('File Type'))

    def __str__(self):
        return self.url


import os

from django.conf import settings
from django.http import FileResponse, Http404
from django.shortcuts import get_object_or_404
from django.views import View

from custom_url.models import CustomUrl

class CustomUrlView(View):
    http_method_names = ['get',]
    
    def get(self, request, *args, **kwargs):
        url = self.kwargs.get('url', None)

        if not url.startswith("/"):
            url = '/' + url
        if not url.endswith("/") and settings.APPEND_SLASH:
            url += '/'
        obj = get_object_or_404(CustomUrl, url=url)
        
        content_type = obj.file_type
        file = obj.file
        file_name = os.path.basename(file.name)

        try:
            return FileResponse(file, content_type=content_type, filename=file_name)
        except:
            raise Http404()